@@ -2,14 +2,16 @@ import asyncio
 import logging
 import os
 import socket
-import sys
-from typing import Any, Iterator, Optional
+from typing import (
+    Any, Iterator, Optional, ParamSpec, Protocol, TypeAlias, final,
+)
 
 from ._context_vars import EVENT_LOOP
 
 
 log = logging.getLogger(__name__)
 
+
 try:
     from time import time_ns
 except ImportError:
@@ -18,28 +20,11 @@ except ImportError:
     def time_ns() -> int:
         return int(time() * 1000000000)
 
-try:
-    from typing import final
-except ImportError:
-    from typing_extensions import final  # type: ignore
-
 
 try:
-    from typing import TypeAlias
+    from typing import Concatenate
 except ImportError:
-    from typing_extensions import TypeAlias
-
-
-if sys.version_info >= (3, 10):
-    from typing import ParamSpec
-else:
-    from typing_extensions import ParamSpec
-
-
-if sys.version_info >= (3, 8):
-    from typing import Protocol
-else:
-    from typing_extensions import Protocol
+    from typing_extensions import Concatenate
 
 
 class EntrypointProtocol(Protocol):
@@ -77,12 +62,13 @@ class EventLoopMixin:
     def loop(self) -> asyncio.AbstractEventLoop:
         if not getattr(self, "_loop", None):
             self._loop = asyncio.get_running_loop()
-        return self._loop   # type: ignore
+        return self._loop  # type: ignore
 
 
 event_loop_policy: asyncio.AbstractEventLoopPolicy
 try:
     import uvloop
+
     if (
         os.getenv("AIOMISC_USE_UVLOOP", "1").lower() in
         ("yes", "1", "enabled", "enable", "on", "true")
@@ -93,7 +79,6 @@ try:
 except ImportError:
     event_loop_policy = asyncio.DefaultEventLoopPolicy()
 
-
 if hasattr(socket, "TCP_NODELAY"):
     def sock_set_nodelay(sock: socket.socket) -> None:
         if sock.proto != socket.IPPROTO_TCP:
@@ -120,6 +105,7 @@ set_current_loop = EVENT_LOOP.set
 get_current_loop = EVENT_LOOP.get
 
 __all__ = (
+    "Concatenate",
     "EntrypointProtocol",
     "EventLoopMixin",
     "ParamSpec",
