@@ -1,5 +1,7 @@
+import asyncio
 import socket
 import ssl
+from contextlib import suppress
 from typing import Any, Dict, Iterable, Mapping, Optional, Tuple, Union
 
 from aiohttp.web import Application, AppRunner, BaseRunner, SockSite  # noqa
@@ -74,14 +76,12 @@ class AIOHTTPService(Service):
         if getattr(self, "runner", None) is None:
             raise RuntimeError("runner already created")
 
-        return SockSite(
-            self.runner, self.socket,
-            shutdown_timeout=self.shutdown_timeout,
-        )
+        return SockSite(self.runner, self.socket)
 
     async def make_runner(self, application: Application) -> AppRunner:
         return AppRunner(
             application,
+            shutdown_timeout=self.shutdown_timeout,
             **self.runner_kwargs,
         )
 
@@ -101,8 +101,12 @@ class AIOHTTPService(Service):
             if self.site:
                 await self.site.stop()
         finally:
+            # Ensure all handlers are finished
+            await asyncio.sleep(0)
             if hasattr(self, "runner"):
-                await self.runner.cleanup()
+                # Avoid AttributeError in case server already dereferenced
+                with suppress(AttributeError):
+                    await self.runner.cleanup()
 
 
 class AIOHTTPSSLService(AIOHTTPService):
@@ -129,7 +133,6 @@ class AIOHTTPSSLService(AIOHTTPService):
 
         return SockSite(
             self.runner, self.socket,
-            shutdown_timeout=self.shutdown_timeout,
             ssl_context=await self.loop.run_in_executor(
                 None, self.__ssl_options.create_context,
             ),
