@@ -2,31 +2,42 @@ import asyncio
 import contextvars
 import inspect
 import logging
+import os
 import threading
 import time
 import warnings
+from abc import ABC, abstractmethod
 from concurrent.futures import ThreadPoolExecutor as ThreadPoolExecutorBase
 from dataclasses import dataclass, field
-from functools import partial, wraps
+from functools import partial
 from multiprocessing import cpu_count
 from queue import SimpleQueue
 from types import MappingProxyType
 from typing import (
-    Any, Awaitable, Callable, Coroutine, Dict, FrozenSet, Optional, Set, Tuple,
-    TypeVar,
+    Any, Awaitable, Callable, Coroutine, Dict, FrozenSet, Generator, Generic,
+    Optional, Set, Tuple, TypeVar, Union, overload,
 )
 
 from ._context_vars import EVENT_LOOP
-from .compat import ParamSpec
+from .compat import Concatenate, ParamSpec
 from .counters import Statistic
 from .iterator_wrapper import IteratorWrapper
 
 
+# ParamSpec for functions
 P = ParamSpec("P")
+# bounded ParamSpec for bound methods
+BP = ParamSpec("BP")
+
 T = TypeVar("T")
+S = TypeVar("S", bound=object)
 F = TypeVar("F", bound=Callable[..., Any])
 log = logging.getLogger(__name__)
 
+THREADED_ITERABLE_DEFAULT_MAX_SIZE = int(
+    os.getenv("THREADED_ITERABLE_DEFAULT_MAX_SIZE", 1024),
+)
+
 
 def context_partial(
     func: F, *args: Any,
@@ -224,9 +235,7 @@ class ThreadPoolExecutor(ThreadPoolExecutorBase):
     def submit(  # type: ignore
         self, fn: F, *args: Any, **kwargs: Any,
     ) -> asyncio.Future:
-        """
-        Submit blocking function to the pool
-        """
+        """Submit blocking function to the pool"""
         if fn is None or not callable(fn):
             raise ValueError("First argument must be callable")
 
@@ -329,6 +338,7 @@ def run_in_executor(
             return await loop.run_in_executor(
                 executor, partial(func, *args, **kwargs),
             )
+
         return lazy_wrapper()
 
 
@@ -342,22 +352,109 @@ async def _awaiter(future: asyncio.Future) -> T:
         raise
 
 
+class ThreadedBase(Generic[P, T], ABC):
+
+    func: Callable[P, T]
+
+    @abstractmethod
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        ...
+
+    def sync_call(self, *args: P.args, **kwargs: P.kwargs) -> T:
+        return self.func(*args, **kwargs)
+
+    def async_call(self, *args: P.args, **kwargs: P.kwargs) -> Awaitable[T]:
+        return run_in_executor(func=self.func, args=args, kwargs=kwargs)
+
+    def __repr__(self) -> str:
+        f = getattr(self.func, "func", self.func)
+        name = getattr(f, "__name__", f.__class__.__name__)
+        return f"<{self.__class__.__name__} {name} at {id(self):#x}>"
+
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> Awaitable[T]:
+        return self.async_call(*args, **kwargs)
+
+
+class Threaded(ThreadedBase[P, T]):
+    func_type: type
+
+    def __init__(self, func: Callable[P, T]) -> None:
+        if isinstance(func, staticmethod):
+            self.func_type = staticmethod
+            self.func = func.__func__
+        elif isinstance(func, classmethod):
+            self.func_type = classmethod
+            self.func = func.__func__
+        else:
+            self.func_type = type(func)
+            self.func = func
+
+        if asyncio.iscoroutinefunction(self.func):
+            raise TypeError("Can not wrap coroutine")
+        if inspect.isgeneratorfunction(self.func):
+            raise TypeError("Can not wrap generator function")
+
+    @overload
+    def __get__(
+        self: "Threaded[Concatenate[S, BP], T]",
+        instance: S,
+        owner: Optional[type] = ...,
+    ) -> "BoundThreaded[BP, T]":
+        ...
+
+    @overload
+    def __get__(
+        self: "Threaded[P, T]",
+        instance: None,
+        owner: Optional[type] = ...,
+    ) -> "Threaded[P, T]":
+        ...
+
+    def __get__(
+        self,
+        instance: Any,
+        owner: Optional[type] = None,
+    ) -> "Threaded[P, T] | BoundThreaded[Any, T]":
+        if self.func_type is staticmethod:
+            return self
+        elif self.func_type is classmethod:
+            cls = owner if instance is None else type(instance)
+            return BoundThreaded(self.func, cls)
+        elif instance is not None:
+            return BoundThreaded(self.func, instance)
+        return self
+
+
+class BoundThreaded(ThreadedBase[P, T]):
+    __instance: Any
+
+    def __init__(self, func: Callable[..., T], instance: Any) -> None:
+        self.__instance = instance
+        self.func = lambda *args, **kwargs: func(instance, *args, **kwargs)
+
+
+@overload
+def threaded(func: Callable[P, T]) -> Threaded[P, T]:
+    ...
+
+
+@overload
 def threaded(
-    func: Callable[P, T],
-) -> Callable[P, Awaitable[T]]:
-    if asyncio.iscoroutinefunction(func):
-        raise TypeError("Can not wrap coroutine")
+    func: Callable[P, Generator[T, None, None]],
+) -> Callable[P, IteratorWrapper[P, T]]:
+    ...
 
+
+def threaded(
+    func: Callable[P, T] | Callable[P, Generator[T, None, None]],
+) -> Threaded[P, T] | Callable[P, IteratorWrapper[P, T]]:
     if inspect.isgeneratorfunction(func):
-        return threaded_iterable(func)
+        return threaded_iterable(
+            func,
+            max_size=THREADED_ITERABLE_DEFAULT_MAX_SIZE,
+        )
 
-    @wraps(func)
-    def wrap(
-        *args: P.args, **kwargs: P.kwargs,
-    ) -> Awaitable[T]:
-        return run_in_executor(func=func, args=args, kwargs=kwargs)
-
-    return wrap
+    return Threaded(func)   # type: ignore
 
 
 def run_in_new_thread(
@@ -392,43 +489,166 @@ def run_in_new_thread(
     return future
 
 
+class ThreadedSeparate(Threaded[P, T]):
+    __slots__ = Threaded.__slots__ + ("detach",)
+
+    def __init__(self, func: Callable[P, T], detach: bool = True) -> None:
+        super().__init__(func)
+        self.detach = detach
+
+    def async_call(self, *args: P.args, **kwargs: P.kwargs) -> Awaitable[T]:
+        return run_in_new_thread(
+            self.func, args=args, kwargs=kwargs, detach=self.detach,
+        )
+
+
 def threaded_separate(
-    func: F,
+    func: Callable[P, T],
     detach: bool = True,
-) -> Callable[..., Awaitable[Any]]:
+) -> ThreadedSeparate[P, T]:
     if isinstance(func, bool):
+        # noinspection PyTypeChecker
         return partial(threaded_separate, detach=detach)
 
     if asyncio.iscoroutinefunction(func):
         raise TypeError("Can not wrap coroutine")
 
-    @wraps(func)
-    def wrap(*args: Any, **kwargs: Any) -> Any:
-        future = run_in_new_thread(
-            func, args=args, kwargs=kwargs, detach=detach,
-        )
-        return future
+    return ThreadedSeparate(func, detach=detach)
 
-    return wrap
+
+class ThreadedIterableBase(Generic[P, T], ABC):
+    func: Callable[P, Generator[T, None, None]]
+    max_size: int
+
+    @abstractmethod
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        ...
+
+    def sync_call(
+        self, *args: P.args, **kwargs: P.kwargs,
+    ) -> Generator[T, None, None]:
+        return self.func(*args, **kwargs)
+
+    def async_call(
+        self, *args: P.args, **kwargs: P.kwargs,
+    ) -> IteratorWrapper[P, T]:
+        return self.create_wrapper(*args, **kwargs)
+
+    def create_wrapper(
+        self, *args: P.args, **kwargs: P.kwargs,
+    ) -> IteratorWrapper[P, T]:
+        return IteratorWrapper(
+            partial(self.func, *args, **kwargs),
+            max_size=self.max_size,
+        )
+
+    def __call__(
+        self,
+        *args: P.args,
+        **kwargs: P.kwargs,
+    ) -> IteratorWrapper[P, T]:
+        return self.async_call(*args, **kwargs)
+
+
+class ThreadedIterable(ThreadedIterableBase[P, T]):
+    func_type: type
+
+    def __init__(
+        self,
+        func: Callable[P, Generator[T, None, None]],
+        max_size: int = 0,
+    ) -> None:
+        if isinstance(func, staticmethod):
+            self.func_type = staticmethod
+            actual_func = func.__func__
+        elif isinstance(func, classmethod):
+            self.func_type = classmethod
+            actual_func = func.__func__
+        else:
+            self.func_type = type(func)
+            actual_func = func
+
+        self.func = actual_func
+        self.max_size = max_size
+
+    @overload
+    def __get__(
+        self: "ThreadedIterable[Concatenate[S, BP], T]",
+        instance: S,
+        owner: Optional[type] = ...,
+    ) -> "BoundThreadedIterable[BP, T]":
+        ...
+
+    @overload
+    def __get__(
+        self: "ThreadedIterable[P, T]",
+        instance: None,
+        owner: Optional[type] = ...,
+    ) -> "ThreadedIterable[P, T]":
+        ...
+
+    def __get__(
+        self,
+        instance: Any,
+        owner: Optional[type] = None,
+    ) -> "ThreadedIterable[P, T] | BoundThreadedIterable[Any, T]":
+        if self.func_type is staticmethod:
+            return self
+        elif self.func_type is classmethod:
+            cls = owner if instance is None else type(instance)
+            return BoundThreadedIterable(self.func, cls, self.max_size)
+        elif instance is not None:
+            return BoundThreadedIterable(self.func, instance, self.max_size)
+        return self
+
+
+class BoundThreadedIterable(ThreadedIterableBase[P, T]):
+    __instance: Any
+
+    def __init__(
+        self,
+        func: Callable[..., Generator[T, None, None]],
+        instance: Any,
+        max_size: int = 0,
+    ) -> None:
+        self.__instance = instance
+        self.func = lambda *args, **kwargs: func(instance, *args, **kwargs)
+        self.max_size = max_size
+
+
+@overload
+def threaded_iterable(
+    func: Callable[P, Generator[T, None, None]],
+    *,
+    max_size: int = 0,
+) -> "ThreadedIterable[P, T]":
+    ...
+
+
+@overload
+def threaded_iterable(
+    *,
+    max_size: int = 0,
+) -> Callable[
+    [Callable[P, Generator[T, None, None]]], ThreadedIterable[P, T],
+]:
+    ...
 
 
 def threaded_iterable(
-    func: Optional[F] = None,
+    func: Optional[Callable[P, Generator[T, None, None]]] = None,
+    *,
     max_size: int = 0,
-) -> Any:
-    if isinstance(func, int):
-        return partial(threaded_iterable, max_size=func)
+) -> Union[
+    ThreadedIterable[P, T],
+    Callable[
+        [Callable[P, Generator[T, None, None]]], ThreadedIterable[P, T],
+    ],
+]:
     if func is None:
-        return partial(threaded_iterable, max_size=max_size)
+        return lambda f: ThreadedIterable(f, max_size=max_size)
 
-    @wraps(func)
-    def wrap(*args: Any, **kwargs: Any) -> Any:
-        return IteratorWrapper(
-            partial(func, *args, **kwargs),
-            max_size=max_size,
-        )
-
-    return wrap
+    return ThreadedIterable(func, max_size=max_size)
 
 
 class IteratorWrapperSeparate(IteratorWrapper):
@@ -436,23 +656,50 @@ class IteratorWrapperSeparate(IteratorWrapper):
         return run_in_new_thread(self._in_thread)
 
 
-def threaded_iterable_separate(
-    func: Optional[F] = None,
-    max_size: int = 0,
-) -> Any:
-    if isinstance(func, int):
-        return partial(threaded_iterable_separate, max_size=func)
-    if func is None:
-        return partial(threaded_iterable_separate, max_size=max_size)
-
-    @wraps(func)
-    def wrap(*args: Any, **kwargs: Any) -> Any:
+class ThreadedIterableSeparate(ThreadedIterable[P, T]):
+    def create_wrapper(
+        self, *args: P.args, **kwargs: P.kwargs,
+    ) -> IteratorWrapperSeparate:
         return IteratorWrapperSeparate(
-            partial(func, *args, **kwargs),
-            max_size=max_size,
+            partial(self.func, *args, **kwargs),
+            max_size=self.max_size,
         )
 
-    return wrap
+
+@overload
+def threaded_iterable_separate(
+    func: Callable[P, Generator[T, None, None]],
+    *,
+    max_size: int = 0,
+) -> "ThreadedIterable[P, T]":
+    ...
+
+
+@overload
+def threaded_iterable_separate(
+    *,
+    max_size: int = 0,
+) -> Callable[
+    [Callable[P, Generator[T, None, None]]],
+    ThreadedIterableSeparate[P, T],
+]:
+    ...
+
+
+def threaded_iterable_separate(
+    func: Optional[Callable[P, Generator[T, None, None]]] = None,
+    *,
+    max_size: int = 0,
+) -> Union[
+    ThreadedIterable[P, T],
+    Callable[
+        [Callable[P, Generator[T, None, None]]], ThreadedIterableSeparate[P, T],
+    ],
+]:
+    if func is None:
+        return lambda f: ThreadedIterableSeparate(f, max_size=max_size)
+
+    return ThreadedIterableSeparate(func, max_size=max_size)
 
 
 class CoroutineWaiter:
@@ -511,4 +758,5 @@ def sync_await(
 ) -> T:
     async def awaiter() -> T:
         return await func(*args, **kwargs)
+
     return wait_coroutine(awaiter())
