@@ -1,13 +1,6 @@
 import asyncio
-import sys
 from functools import wraps
-from typing import Awaitable, Callable, TypeVar, Union
-
-
-if sys.version_info >= (3, 10):
-    from typing import ParamSpec
-else:
-    from typing_extensions import ParamSpec
+from typing import Any, Callable, Coroutine, ParamSpec, TypeVar, Union
 
 
 T = TypeVar("T")
@@ -16,11 +9,14 @@ Number = Union[int, float]
 
 
 def timeout(
-    value: Number
-) -> Callable[[Callable[P, Awaitable[T]]], Callable[P, Awaitable[T]]]:
+    value: Number,
+) -> Callable[
+    [Callable[P, Coroutine[Any, Any, T]]],
+    Callable[P, Coroutine[Any, Any, T]],
+]:
     def decorator(
-        func: Callable[P, Awaitable[T]],
-    ) -> Callable[P, Awaitable[T]]:
+        func: Callable[P, Coroutine[Any, Any, T]],
+    ) -> Callable[P, Coroutine[Any, Any, T]]:
         if not asyncio.iscoroutinefunction(func):
             raise TypeError("Function is not a coroutine function")
 
